#include <stdio.h>
#include "winbindd.h"

#define WINBIND_IDMAP_DIR "/usr/local/samba/etc"
#define WINBIND_IDMAP_UIDFILE WINBIND_IDMAP_DIR "/uidmap"
#define WINBIND_IDMAP_GIDFILE WINBIND_IDMAP_DIR "/gidmap"

BOOL flatfile_finduid(uid_t *uid, fstring dom_name, fstring name);
BOOL flatfile_findgid(uid_t *uid, fstring dom_name, fstring name);

struct winbindd_idmap_ops flatfile_idmap_ops = {
    flatfile_finduid,
    flatfile_findgid
};


struct winbindd_idmap_ops *idmap_init(int *version) {
    *version = WINBINDD_IDMAP_INTERFACE_VERSION;
    return &flatfile_idmap_ops;
}

static BOOL flatfile_searchfile(char *path, uid_t *uid, fstring dom_name, fstring name) {
    FILE *f;
    char buf[BUFSIZ];
    char *tmpd, *tmpn, *tmpi, *comment;
    int line=0;

    f = fopen(path, "r");
    if ( f == NULL ) {
        DEBUG(0, ("Cannot open '%s': %s\n", path, strerror(errno)));
        return False;
    }

    while ( fgets(buf, BUFSIZ, f) ) {

        line++;
        /* strip comments and any white space that appears before them */
        comment = strtok(buf, "#");
        comment = strtok(NULL, "#");
        if ( comment ) {
            *comment = '\0';
            for ( comment-- ; isspace(*comment) ; comment-- ) {
                *comment = '\0';
            }
        }
        DEBUG(10, ("Line %d sans comments: '%s'\n", line, buf));

        /* find user, terminate domain */
        tmpd = strtok(buf, ":");
        if ( tmpd == NULL ) {
            DEBUG(1, ("%s:%d: error fetching DOMAIN token\n",
                        path, line));
            continue;
        }

        tmpn = strtok(NULL, ":");
        if (tmpn == NULL ) {
            DEBUG(1, ("%s:%d: error fetching USER/GROUP token\n",
                        path, line));
            continue;
        }

        tmpi = strtok(NULL, " \t\r\n");
        if (tmpi == NULL ) {
            DEBUG(1, ("%s:%d: error fetching UID/GID token\n",
                        path, line));
            continue;
        }

        /* Be sure domain matches before continuing */
        if ( strcmp(dom_name, tmpd) ) {
            continue;
        }

        /* Be sure domain matches before continuing */
        if ( strcmp(name, tmpn) ) {
            continue;
        }

        *uid = atoi(tmpi);
        fclose(f);

        DEBUG(4, ("found a match.  Returning id %d from string '%s'\n",
                    *uid, tmpi));

        return True;
    }

    fclose(f);
    DEBUG(4, ("lookup failed for '%s\\%s' in %s\n", dom_name, name, path));

    return False;
}

BOOL flatfile_finduid(uid_t *uid, fstring dom_name, fstring name) {
    return flatfile_searchfile(WINBIND_IDMAP_UIDFILE, uid, dom_name, name);
}

BOOL flatfile_findgid(uid_t *gid, fstring dom_name, fstring name) {
    return flatfile_searchfile(WINBIND_IDMAP_GIDFILE, gid, dom_name, name);
}

